package com.example.teacher.service;

import com.example.teacher.dto.TeacherDto;
import com.example.teacher.exception.ResourceNotFoundCustomException;
import com.example.teacher.mapper.TeacherMapper;
import com.example.teacher.model.Teacher;
import com.example.teacher.repository.TeacherRepository;
import io.restassured.RestAssured;
import org.assertj.core.api.DateAssert;
import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.jupiter.api.AfterEach;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

//@RunWith(MockitoJUnitRunner.class)
public class TeacherServiceTest {

    @Mock
    private TeacherRepository teacherRepository;

    @Mock
    private TeacherMapper teacherMapper;

    @InjectMocks
    private TeacherService teacherService;

    private Teacher myTeacher;
    private List<Teacher> teacherList;
    private TeacherDto myTeacherDto;
    private Date date;

    AutoCloseable openMocks;

    @Before
    public void setup() {
        // initializes a mock object for those fields that have @Mock on them.
        openMocks = MockitoAnnotations.openMocks(this);
    }

    @Before
    public void createTeacher() throws ParseException {
        // create a dummy object
        date = new SimpleDateFormat("yyyy:MM:dd").parse("11:11:2011");
        myTeacher = new Teacher(0, "Teacher", 25, "maths", date, true);
        myTeacherDto = new TeacherDto(0, "Teacher", 25, "maths", date, null);
        teacherList = new ArrayList<>();
        teacherList.add(myTeacher);
    }

    @Test
    public void getAllTeachersTest() {
        // run this instead of calling the actual repository method
        Mockito.when(teacherRepository.findAll()).thenReturn(teacherList);
        List<Teacher> mockTeacherList = teacherService.findAllTeachers();

        // assert statements checks if the results are as we expected.
        Assert.assertEquals(mockTeacherList.get(0), teacherList.get(0));
    }

    @Test
    public void findTeacherByIdTest() throws InterruptedException, ResourceNotFoundCustomException {
        Mockito.when(teacherRepository.findById(0)).thenReturn(Optional.of(myTeacher));
        Optional<Teacher> returnedTeacher = teacherService.findTeacherById(0);
        Assert.assertEquals(returnedTeacher.get(), myTeacher);
    }

    @Test

    public void findTeacherByWrongIdTest() throws InterruptedException {
        Mockito.when(teacherRepository.findById(0)).thenReturn(Optional.empty());
        try {
            Optional<Teacher> returnedTeacher = teacherService.findTeacherById(0);
        }
        catch (ResourceNotFoundCustomException e) {
            Assert.assertEquals(e.getMessage(), "Teacher does not exist");
        }
    }

    @Test
    public void findTeacherByWrongIdTest1() throws InterruptedException {
        Mockito.when(teacherRepository.findById(0)).thenReturn(Optional.empty());
        Exception exception = Assert.assertThrows(ResourceNotFoundCustomException.class, () -> teacherService.findTeacherById(0));
        Assert.assertEquals(exception.getMessage(), "Teacher does not exist");
    }

    @Test(expected = ResourceNotFoundCustomException.class)
    public void findTeacherByWrongIdTest2() throws InterruptedException, ResourceNotFoundCustomException {
        Mockito.when(teacherRepository.findById(0)).thenReturn(Optional.empty());
        Optional<Teacher> returnedTeacher = teacherService.findTeacherById(0);
    }

    @Test
    public void findTeacherByNameTest() throws ParseException {
        Mockito.when(teacherRepository.findTeacherByName("Teacher")).thenReturn(myTeacher);
        Mockito.when(teacherMapper.entityToDto(myTeacher)).thenReturn(myTeacherDto);
        TeacherDto returnedTeacher = teacherService.findTeacherByName("Teacher");
        Assert.assertEquals(returnedTeacher.getName(), myTeacherDto.getName());
        Assert.assertEquals(returnedTeacher.getAge(), myTeacherDto.getAge());
//        // assertEquals will check the reference of the object instead of its attributes.
//        Assert.assertEquals(returnedTeacher, myTeacherDto);
        Assert.assertEquals(returnedTeacher.getDateOfJoining(), date);
    }

    @Test
    public void findTeacherByAgeTest() {
        Mockito.when(teacherRepository.findTeacherByAge(25)).thenReturn(teacherList);
        List<Teacher> returnedTeacherList = teacherService.findTeacherByAge(25);
        Assert.assertEquals(returnedTeacherList, teacherList);
    }

    @Test
    public void deleteTeacherByIdTest() {
        Teacher myDeletedTeacher = new Teacher(0, "Teacher", 25, "maths", new Date(), false);
        Mockito.when(teacherRepository.findTeacherById(0)).thenReturn(myTeacher);
        boolean isDeleted = teacherService.deleteTeacherById(0);
        Assert.assertTrue(isDeleted);
    }

    @Test
    public void deleteTeacherByWrongIdTest() {
        Teacher myDeletedTeacher = new Teacher(0, "Teacher", 25, "maths", new Date(), false);
        Mockito.when(teacherRepository.findTeacherById(0)).thenReturn(null);
        boolean isDeleted = teacherService.deleteTeacherById(0);
        Assert.assertFalse(isDeleted);
    }

    @Test
    public void createTeacherTest() throws JSONException {
        Date date = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MM-yyyy");
        String stringDate = simpleDateFormat.format(date);
        JSONObject requestParams = new JSONObject();
        requestParams.put("name", "Testname");
        requestParams.put("age", 25);
        requestParams.put("subject", "maths");
        requestParams.put("dateOfJoining", stringDate);
    }

    @AfterEach
    public void tearDown() throws Exception {
        openMocks.close();
    }
}